# Core Data

[![Build Status](https://travis-ci.org/nodewrite/nodewrite-core-data.svg?branch=master)](https://travis-ci.org/nodewrite/nodewrite-core-data)
[![CodeClimate](https://codeclimate.com/github/nodewrite/nodewrite-core-data/badges/gpa.svg)](https://codeclimate.com/github/nodewrite/nodewrite-core-data)
[![Coverage Status](https://coveralls.io/repos/github/nodewrite/nodewrite-core-data/badge.svg?branch=master)](https://coveralls.io/github/nodewrite/nodewrite-core-data?branch=master)
[![Dependency Status](https://gemnasium.com/badges/github.com/nodewrite/nodewrite-core-data.svg)](https://gemnasium.com/github.com/nodewrite/nodewrite-core-data)

Handles registering plugin data models and establishing connections with the
database. Provides a datastore-agnostic JavaScript ORM designed for
ease of use and peace of mind. Exposes a unified data API that works on both
server and client.

## Installation
```shell
npm install nodewrite-core-data --save
```
> This is a core package and is installed by default.

## Usage
Provides access to data store models that have been registered by plugins.

### Plugins
When developing a plugin that depends on this package:
```js
server.dependency(['nodewrite-core-data'], register);
```
Learn more about [how to build a plugin]().

## Helper Methods
* [`server.store`](#serverstore)
* [`server.paginate(data)`](#serverpaginatedata)
* [`server.registerPluginModels(path)`](#serveregisterpluginmodelspath)

### `server.store()`
Plugin models are each assigned to their own namespace within the store. The
store contains a top-level namespace for both core and plugin packages.
```js
// Get reference to data store.
const store = server.store();

// store.Core.[Package].[Model]
const User = store.Core.Users.User;

// store.Plugin.[Package].[Model]
const Post = store.Plugin.Posts.Post;
```
Learn more about [how to use JSData models](http://www.js-data.io/docs/ds).

### `server.paginate(data)`
The paginate method will return a standardized pagination object when given
the total number of items, items per page and current page.
* `data` - an object used for generating the pagination object.
  * `total` - total number of items in the data set.
  * `limit` - total number of items per page.
  * `page` - current page position.
```js
const data = { total: 100, limit: 15, page: 3 };
const pagination = server.paginate(data);
```
Resulting pagination object:
```js
{
  currentPage: 3,
  itemsPerPage: 15,
  nextPage: 4,
  previousPage: 2,
  totalItems: 100,
  totalPages: 7
}
```

### `server.registerPluginModels(path)`
Registers plugin models with the data store. The models will be expected to be
found within the plugin's `/model` directory. Registration of plugin models may
become automatic in the future.
* `path` - base directory of a plugin that has data models to be registered.
```js
server.registerPluginModels(__dirname);
```

## Getting Help

Open an issue on this repository!
