'use strict';

const { container, register } = require('nodewrite-core-container');
const plugin = require(process.cwd());

container.resolve((Hapi, test, path) => {

  const { lab, expect, describe, it, start, stop } = test;

  module.exports.lab = lab;

  describe(plugin.register.attributes.pkg.name, () => {

    const server = new Hapi.Server();

    lab.before((done) => start({ server, container, register, plugin, done }));
    lab.after((done) => stop({ server, done }));

    it('Decorates server with a method for accessing the data store.', (done) => {

      expect(server.store).to.be.a.function();
      done();
    });

    it('Decorates server with a method for registering plugin models.', (done) => {

      expect(server.registerPluginModels).to.be.a.function();
      done();
    });

    it('Decorates server with a method for generating pagination objects.', (done) => {

      expect(server.paginate).to.be.a.function();
      done();
    });

    it('Registers models when passed a plugin package directory.', (done) => {

      expect(() => {

        server.registerPluginModels(path.join(__dirname, 'fixtures', 'nodewrite-plugin-test'));
      }).to.not.throw();
      done();
    });

    it('Registers models when passed a core package directory.', (done) => {

      expect(() => {

        server.registerPluginModels(path.join(__dirname, 'fixtures', 'nodewrite-core-test'));
      }).to.not.throw();
      done();
    });

    it('Returns pagination object containing the expected keys.', (done) => {

      expect(server.paginate({ limit: 10, total: 9, page: 1 })).to.include([
        'currentPage',
        'itemsPerPage',
        'nextPage',
        'previousPage',
        'totalItems',
        'totalPages'
      ]);
      done();
    });

    it('Properly calculates values of the expected pagination object keys.', (done) => {

      const pagination = (page) => server.paginate({ limit: 10, total: 100, page });
      expect(pagination(5).currentPage).to.equal(5);
      expect(pagination(5).itemsPerPage).to.equal(10);
      expect(pagination(1).nextPage).to.equal(2);
      expect(pagination(2).previousPage).to.equal(1);
      expect(pagination(5).totalItems).to.equal(100);
      expect(pagination(5).totalPages).to.equal(10);
      done();
    });

  });

});
