'use strict';

const DotProp = require('dot-prop');
const OmitBy = require('lodash/omitBy');
const IsNil = require('lodash/isNil');
const Merge = require('lodash/merge');

module.exports = function (chunk, context, bodies, params) {

  const defaults = {
    as: undefined,
    id: undefined,
    model: undefined,
    limit: 5,
    offset: 0,
    orderBy: undefined,
    sortOrder: 'ASC'
  };

  const options = Merge(defaults, OmitBy({
    as: context.resolve(params.as),
    id: context.resolve(params.id),
    model: context.resolve(params.model),
    limit: context.resolve(params.limit),
    offset: context.resolve(params.offset),
    orderBy: context.resolve(params.orderBy),
    sortOrder: context.resolve(params.sortOrder)
  }, IsNil));

  const request = context.get('request');
  const store = request.server.store();
  const data = DotProp.get(store, options.model);

  if (options.id) {
    return data.find(options.id);
  }

  const filter = OmitBy({
    limit: options.limit,
    offset: options.offset
  }, IsNil);

  if (options.orderBy && options.sortOrder) {
    filter.orderBy = [[options.orderBy, options.sortOrder]];
  }

  return data.findAll(filter)
    .then((resources) => {

      return { [options.as]: resources };
    });
};
